<div style="text-align:center"><img src="./image/loris.jpg" /></div>

# Python Application Logging Module 

The centralized application logging library.

# Logging Data Level

- **`FATAL`**: Log the application/ solution/ API state or behaviours that causes the system to terminate unintentionally; or have to cause the system to halt.
- **`ERROR`**: Log the application/ solution/ API state or behaviours that causes the system response abnormally; or is returning not expected out; but the system is still running.
- **`DEBUG`**: Log the application/ solution/ API behaviours or state change.
- **`INFO`**: Log the AI model performance or output.

# Unique ID Generation

Please do use uuid4 for generating the random unique ID.

```python 
import uuid 
from loris_log.boto3Client import Boto3Client

# generating the random uu_id for each calling session.
id = uuid.uuid4()
```

Otherwise use uuid5 for fixed id generation.

```python
import uuid 

# generating the fixed uu_id for each calling session.
id = uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
```

# Log Message onto CloudWatch Examples

## Construct the Error Log Message

This shows the example of constructing the error log message for CloudWatch.

**`def set_error_message(uu_id, name, message)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of an application/ remote host IP address/ service.
- **`message (list)`**: The error message.

```python
from loris_log.customLog import CustomLog
import uuid

# get the random uuid generated
id = uuid.uuid4()
# define the error log message as the string list.
error_message = ["[400]", "Service not found."]
# define the name (string), oftenly is the IP address of remote host, or the application name
name = "192.168.0.0"

# then, construct the corresponding error log message by initialize set_error_message
# function, and passing in the id (uuid::UUID), name (string), and error_message (list).
error_log = CustomLog.set_error_message(id, name, error_message)
```

## Construct the Debug Log Message

This shows the example of constructing the debug log message for CloudWatch. 

**`def set_debug_message(uu_id, name, message)`:**


Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of the application/ remote host IP address/ service.
- **`message (list)`**: The debug message.


```python
from loris_log.customLog import CustomLog
import uuid 

# get the random uuid
id = uuid.uuid4()
# define the debug message as the string list
debug_message = ["[200]", "Service ok"]
# define the name (string), oftenly is the IP address of remote host, or the application name
name = "192.168.0.0"

# then, construct the corresponding debug log message by calling set_debug_message function,
# and passing in the id (uuid::UUID), name (string), and debug_message (string).
debug_log = CustomLog.set_debug_message(id, name, debug_message)
```

## Construct the Fatal Log Message

This shows the example of constructing the fatal log message for CloudWatch.

**`def set_fatal_message(uu_id, name, message)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of the application/ remote host IP address/ service.
- **`message (list)`**: The fatal message.

```python

from loris_log.customLog import CustomLog 
import uuid 

# get the random uuid
id = uuid.uuid4()
# define the fatal message as the string list 
fatal_message = ["[500", "Service terminated unexpectedly"]
# define the name (string), oftenly is the IP address of remote host, or the application name.
name = "192.168.0.0"

# then, construct the corresponding fatal log message by calling the set_fatal_message
# function and passing it id (uuid::UUID), name (string), and message (list).
fatal_log = CustomLog.set_fatal_message(id, name, fatal_message)
```

## Construct the Info Log Message

This shows the example of constructing the info log message for CloudWatch.

**`def set_info_message(uu_id, name, message)`**

Arguments:

- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of the application/ remote IP address/ service.
- **`message (list)`**: The info message.

```python
from loris_log.customLog import CustomLog 
import uuid 

# get the random uuid generated.
id = uuid.uuid4()
# define the fatal message as the string list.
info_message = ["[200]", "the accuracy is 0.99"]
# define the name (string), oftenly is the IP address of remote host, or the application name.
name = "192.168.0.0"

# then, construct the corresponding info log message via calling set_info_message function.
# by passing in the id (uuid::UUID), name (string), and example_fatal_message (string).
info_log = CustomLog.set_info_message(id, name, fatal_message)

```


## Pushing Log Message onto the CloudWatch 

This shows the example of establishing the connection to the AWS CloudWatch service via the following method:

**`def Boto3Client(region_name, aws_key, aws_secret)`**

Arguments:
- **`region_name (string)`**: The AWS service region name. Refers here [region_name](https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Concepts.RegionsAndAvailabilityZones.html).
- **`aws_key (string)`**: The AWS service role access key.
- **`aws_secret (string)`**: The AWS service role secret key.

Then, pushing the constructed message onto the CloudWatch via the `set_log_message` function, after defining its log group and log stream:

**`def set_log_message(log_group_name, log_stream_name, log_message)`**

Arguments:
- **`log_group_name (string)`**: The name of the log group.
- **`log_stream_name (string)`**: The name of the log stream.
- **`log_message (list)`**: A list of application/ service/ api message.

Below shows the complete example of pushing an log message onto the AWS CloudWatch.


```python
import uuid
from loris_log.customLog import CustomLog
from loris_log.boto3Client import Boto3Client

# constuct the log message uses the above elaborated approach. The example
# here creates a sample fatal log message.
# ========================================================================
# get the random uuid
id = uuid.uuid4()
# define the fatal message as the string list 
fatal_message = ["[500]", "Service terminated unexpectedly"]
# define the log name (string), oftenly is the IP address of remote host, or the application name.
name = "192.168.0.0"

# then, construct the corresponding fatal log message
fatal_log = CustomLog.set_fatal_message(id, name, fatal_message)

# Subsequently, push the created fatal log message onto the Cloudwatch
# ###############################################################################
# initialize the boto3 client by first define the log group(string), log stream (string), region (string),
# as well as the key (string) and secret (string). All in form of string.
log_group = ""
log_stream = ""
aws_region = ""
aws_key = ""
aws_secret = ""
# establish the boto3 connection by initialize Boto3Client instance, then
# passing the aws_region (string), aws_key (string) and aws_secret (string).
client = Boto3Client(aws_region, aws_key, aws_secret)
# then push the log message via following set_log_message method by passing them
# log_group name (string), log_stream name (string, and the log_messsage (list).
client.set_log_message(log_group, log_stream, fatal_log)
```

# Log Message onto Remote Server Via FTP Examples

## Construct the Debug Log Message

This shows the example of constructing the debug log message onto the remote FTP server using `set_debug_message` function.

**`def set_debug_message(uu_id, name, message)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of an application/ remote host IP address/ service.
- **`message (list)`**: The debug message.

```python
from loris_log.customLog import CustomLog 

# randomly generated an id using uuid library
id = uuid.uuid4()
# then define the debug log message that is a list
debug_log = ["[200]", "OK"]
# and define the log name that is a string, which usually is the application name
# or the remote host IP address.
name = "192.168.1.1" 

# afterwards, build the complete debug log message string using set_debug_message
# function by passing the id (id::UUID), name (string), and debug_log (list).
debug_message = CustomLog.set_debug_message(id, name, debug_log)
```

## Construct the Info Log Message

This shows the example of constructing the info log message onto the remmote FTP server via the `set_info_message` function.

**`def set_info_message(uu_id, name, message)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of an application/ remote host IP address/ service.
- **`message (list)`**: The info message.

```python
from loris_log.customLog import CustomLog
import uuid 

# get the random uuid generated
id = uuid.uuid4()
# define the info log message as the string list
info_message = ["[200]", "result is 0.9"]
# define the name (string), oftenly is an IP address of remote host, or the application name
name = "192.168.0.0"

# then, construct the corresponding info log message by initialize the set_info_message
# function, and passing in the id (uuid::UUID), name (string), and error_message(list)
info_log = CustomLog.set_info_message(id, name, info_message)
```

## Construct the Error Log Message

This shows the example of constructing the error log message onto the FTP remote server via the `set_error_message` function.

`def set_error_message(uu_id, name, message)`

Arguments:

- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of an application/ remote host IP address/ service.
- **`message (list)`**: The error message.

```python
from loris_log.customLog import CustomLog
import uuid

# get the random uuid generated
id = uuid.uuid4()
# define the error log message as the string list.
error_message = ["[400]", "Service not found."]
# define the name (string), oftenly is the IP address of remote host, or the application name
name = "192.168.0.0"

# then, construct the corresponding error log message by initialize set_error_message
# function, and passing in the id (uuid::UUID), name (string), and error_message (list).
error_log = CustomLog.set_error_message(id, name, error_message)
```

## Construct the Fatal Log Message

This shows the example of constructing the fatal log message for CloudWatch via the `set_fatal_message(uu_id, name, message)` function.

**`def set_fatal_message(uu_id, name, message)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of application/ remote host IP address/ service.
- **`message (list)`**: The fatal message.

```python

from loris_log.customLog import CustomLog 
import uuid 

# get the random uuid
id = uuid.uuid4()
# define the fatal message as the string list 
fatal_message = ["[500", "Service terminated unexpectedly"]
# define the name (string), oftenly is the IP address of remote host, or the application name.
name = "192.168.0.0"

# then, construct the corresponding fatal log message by calling the set_fatal_message
# function and passing it id (uuid::UUID), name (string), and message (list).
fatal_log = CustomLog.set_fatal_message(id, name, fatal_message)
```

## Construct the Log for the Sensor Data

This shows the example of constructing the log message from some embedded device data via the `set_ftp_log_data` function.

**`def set_ftp_log_data(uu_id, starttime, endttime, result, groundtruth)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`starttime (string)`**: The start time of an operation. Conversion from `datetime.datetime.now()`.
- **`endtime (string)`**: The end time of an operation. Conversion from `datetime.datetime.now()`.
- **`result (string)`**: The result of the embedded device/ system/ model etc. Conversion from `double\ float\ long`.
- **`groundtruth (string)`**: The result of embedded device/ system/ model etc. Conversion from `double\ float\ long`.

```python
import datetime 
import uuid
from loris_log.customLog import CustomLog

# generate the uuid
id = uuid.uuid4()
# define the start time
starttime = datetime.datetime.now().strftime("%H:%M:%S")
# define the end time
endtime = datetime.datetime.now().strftime("%H:%M:%S")
# define the result
result = str(32)
# define the groundtruth
groundtruth = str(3)

# construct the log data message
data_log = CustomLog.set_ftp_log_data(id, starttime, endtime, result, groundtruth)
```

# Pushing Info Log Message onto Remote FTP Server

This example shows the approach to push the log message onto the remote FTP server. It first involves construction of the relevant log message. This example, for instance, create the info log message via the `set_info_message` function. 

**`def set_info_message(uu_id, name, message)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`name (string)`**: The name of an application/ remote host IP address/ service.
- **`message (list)`**: The info message.

```python
from loris_log.customLog import CustomLog
import uuid 

# get the random uuid generated
id = uuid.uuid4()
# define the info log message as the string list
info_message = ["[200]", "result is 0.9"]
# define the name (string), oftenly is an IP address of remote host, or the application name
name = "192.168.0.0"

# then, construct the corresponding info log message by initialize the set_info_message
# function, and passing in the id (uuid::UUID), name (string), and error_message(list)
info_log = CustomLog.set_info_message(id, name, info_message)

```

Then, attempt to establish the communication with the target remote FTP server by providing the FTP server `hostname (string)`, `port number (string)`, `username (string)`, as well as the `password (string)` via the `FTPClient` instance

**`def FTPClient(hostname, port_num, username, password)`**

Arguments:
- **`hostname (string)`**: FTP server's hostname.
- **`port_num (integer)`**: FTP server's port number.
- **`username (string)`**: FTP server's username.
- **`password (string)`**: FTP server's password.

as elaborated in the code snippet below.

```python
from loris_log.ftpClient import FtpClient

hostname = ""
port_num = 22
username = ""
password = ""

# Establish the connection to the remote FTP server
client = FtpClient(hostname, port_num, username, password)
```

After the connection was successfully established. It is time to publish the corresponding log message onto the remote FTP server, based on the define file path, and filename via the `set_ftp_log_file` method.

**`def set_ftp_log_file(directory_name, file_name, message)`**

Arguments:
- **`directory_path (string)`**: The path to the log file.
- **`file_name (string)`**: The log file name.
- **`message (string)`**: The log message

```python
# define the path for the log file, must follow the defined format
file_path = "log/testing/data"
# then define the log file name and its file extensions
filename = "sample.txt"

# Push the log file onto the remote FTP server. New log will
client.set_ftp_log_file(file_path, filename, info_log)
```

This shows the complete code example

```python
from loris_log.customLog import CustomLog
from loris_log.ftpClient import FtpClient
import uuid

# get the random uuid generated
id = uuid.uuid4()
# define the info log message as the string list
info_message = ["[200]", "result is 0.9"]
# define the name (string), oftenly is an IP address of remote host, or the application name
name = "192.168.0.0"
# then, construct the corresponding info log message by initialize the set_info_message
# function, and passing in the id (uuid::UUID), name (string), and error_message(list)
info_log = CustomLog.set_info_message(id, name, info_message)

# define the FTP server hostname
hostname = ""
# define the FTP server port number
port_num = 22
# define the FTP server username 
username = ""
# define the FTP server password
password = ""
# Establish the connection to the remote FTP server
client = FtpClient(hostname, port_num, username, password)


# define the path for the log file, must follow the defined format
file_path = "log/testing/data"
# then define the log file name and its file extensions
filename = "sample.txt"
# Push the log file onto the remote FTP server. New log will
client.set_ftp_log_file(file_path, filename, info_log)
```

Please uses `set_error_message`, `set_info_message`, `set_fatal_message`, or `set_debug_message` method to log the `error`, `info`, `fatal` or `debug` message.

# Example of Pushing Sensor Data to Remote FTP Server

This example shows the example of pushing sensor data as log message onto the remote FTP server.

First, construct the log message using the sensor data via the `set_ftp_log_data` method.

**`def set_ftp_log_data(uu_id, starttime, endtime, result, groundtruth)`**

Arguments:
- **`uu_id (uuid::UUID)`**: The generated uuid.
- **`starttime (string)`**: The start time of an operation. Conversion from datetime.datetime.now().
- **`endtime (string)`**: The end time of an operation. Conversion from datetime.datetime.now().
- **`result (string)`**: The result of the sensors, embedded device, system, model etc. Conversion from double/ float/ long.
- **`groundtruth (string)`**: The result of the sensors, embedded device, system, model etc. Conversion from double/ float/ long.

```python
import datetime 
import uuid
from loris_log.customLog import CustomLog

# generate the uuid
id = uuid.uuid4()
# define the start time
starttime = datetime.datetime.now().strftime("%H:%M:%S")
# define the end time
endtime = datetime.datetime.now().strftime("%H:%M:%S")
# define the result
result = str(32)
# define the groundtruth
groundtruth = str(3)

# construct the log data message
data_log = CustomLog.set_ftp_log_data(id, starttime, endtime, result, groundtruth)
```

Once the log message was crafted, it is time to attempt to establish communication with the remote FTP server using the `FTPClient` method.

**`def FTPClient(hostname, port_num, username, password)`**

Arguments:
- **`hostname (string)`**: FTP server's hostname.
- **`port_num (integer)`**: FTP server's port number.
- **`username (string)`**: FTP server's username.
- **`password (string)`**: FTP server's password.

as elaborated in the code snippet below.

```python
from loris_log.ftpClient import FtpClient

hostname = ""
port_num = 22
username = ""
password = ""

# Establish the connection to the remote FTP server
client = FtpClient(hostname, port_num, username, password)
```

Then, defining the path and filename of the corresponding log file, and have the data pushing onto an log file of a FTP server using the `set_ftp_log_data` method.

**`def set_ftp_log_data(directory_path, file_name, log_data)`**

Arguments:
- **`directory_path (string)`**: The path to the log file.
- **`file_name (string)`**: The name of the log file.
- **`log_data (byte)`**: The data to be logged.

```python
# define the path to the log file
directory_path = "/log_testing/data"
# define the log file with its extension
file_name = "log.csv"
# push the log data onto the remote FTP server.
client.set_ftp_log_data(directory_path, file_name, data_log)
```

The complete example is as demonstrated below.

```python
import datetime 
import uuid
from loris_log.customLog import CustomLog
from loris_log.ftpClient import FtpClient

# generate the uuid
id = uuid.uuid4()
# define the start time
starttime = datetime.datetime.now().strftime("%H:%M:%S")
# define the end time
endtime = datetime.datetime.now().strftime("%H:%M:%S")
# define the result
result = str(32)
# define the groundtruth
groundtruth = str(3)
# construct the log data message
data_log = CustomLog.set_ftp_log_data(id, starttime, endtime, result, groundtruth)

# define FTP server hostname
hostname = ""
# define FTP server port number
port_num = 22
# define FTP server username
username = ""
# define FTP server password
password = ""
# Establish the connection to the remote FTP server
client = FtpClient(hostname, port_num, username, password)

# define the path to the log file
directory_path = "/log_testing/data"
# define the log file with its extension
file_name = "log.csv"
# push the log data onto the remote FTP server.
client.set_ftp_log_data(directory_path, file_name, data_log)

```